/**
 * 
 */
package com.as.lyons.wcf.svc.rtn;

import java.util.Iterator;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.soap.Detail;
import javax.xml.soap.DetailEntry;
import javax.xml.soap.SOAPFault;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.helpers.MessageFormatter;
import org.springframework.util.Assert;

import com.as.lyons.wcf.svc.rtn.client.ErrorResult;
import com.as.lyons.wcf.svc.rtn.client.IRTNService;
import com.as.lyons.wcf.svc.rtn.client.LogonResult;
import com.sun.xml.ws.fault.ServerSOAPFaultException;

/**
 * @author kredding
 *
 */
public abstract class AbstractRtnServiceCall implements RtnServiceCall {
	protected static final Logger log = LoggerFactory.getLogger(AbstractRtnServiceCall.class);
	private static Class<?>[] errorClasses = new Class[] { ErrorResult.class };

	protected void handleError(String msg, String errorMessage, boolean rethrow) throws StopProcesingException {
		String resMsg = MessageFormatter.arrayFormat("{}{}{}", new String[] { msg, errorMessage == null ? " - " : ": ",
				errorMessage == null ? "got a SOAP exception" : errorMessage }).getMessage();
		System.out.println(resMsg);
		if (rethrow) {
			throw new StopProcesingException();
		}
	}

	protected void message(String template, Object... args) {
		String msg = MessageFormatter.arrayFormat(template, args).getMessage();
		System.out.println(msg);
	}

	protected String login(IRTNService svc, LogonProperties loginProps) throws StopProcesingException {
		log.info("Login from dinesh");
		String token = null;
		try {
			LogonResult resObj = svc.logon(loginProps.getCompanyId(), loginProps.getUserName(),
					loginProps.getPassword());
			Assert.notNull(resObj, "The returned LogonResult is null");
			token = resObj.getToken();
			Assert.notNull(token, "The returned token is null");
		} catch (ServerSOAPFaultException ex) {
			handleFault("\nError logging on to IRTNService", ex, true);
			log.error("Error logging on to IRTNService", ex);
		} catch (Exception e) {
			handleError("\nError communicating to IRTNService", e.getMessage(), true);
			log.error("Error communicating to IRTNService", e);
		}
		return token;
	}

	protected void logoff(IRTNService svc, String token) {
		try {
			svc.logoff(token);
		} catch (Exception e) {
			// swallowing the exception
		}
	}

	@SuppressWarnings("rawtypes")
	protected void handleFault(String msg, ServerSOAPFaultException ex, boolean rethrow) throws StopProcesingException {
		String res = null;
		SOAPFault fault = ex.getFault();
		if (fault != null) {
			Detail det = fault.getDetail();
			if (det != null) {
				Iterator resItr = det.getDetailEntries();
				if (resItr != null && resItr.hasNext()) {
					DetailEntry de = (DetailEntry) resItr.next();
					if (de != null) {
						try {
							JAXBContext ctx = JAXBContext.newInstance(errorClasses);
							Unmarshaller u = ctx.createUnmarshaller();
							JAXBElement<ErrorResult> resEl = u.unmarshal(de, ErrorResult.class);
							ErrorResult resO = resEl.getValue();
							res = resO.getErrorMessage();
						} catch (JAXBException e) {
						}
					}
				}
			}
		}
		handleError(msg, res, rethrow);
	}
    protected void printHelp(String methodName, StringBuilder message) {
        System.out.println("\"The arguments provided to method is not recognized!\"");		
		StringBuilder sb = new StringBuilder(500).append("To resolve this Use following arguments for method ").append(methodName);
		System.out.println(sb.append(message).toString());		
    }
}
