/**
 * 
 */
package com.as.lyons.wcf.svc.rtn;

import java.util.HashMap;
import java.util.Map;

import org.springframework.util.Assert;

import com.as.lyons.wcf.svc.rtn.client.BooleanResult;
import com.as.lyons.wcf.svc.rtn.client.IRTNService;
import com.sun.xml.ws.fault.ServerSOAPFaultException;

/**
 * @author kredding
 *
 */
public class BooleanResultServiceCall extends AbstractRtnServiceCall {

	private static interface ServiceMethod {
		BooleanResult call(IRTNService svc, String token, String rtn, String Country, Boolean includeInactives,
				Boolean includeWireSearch, Boolean includeHistory)
				throws ServerSOAPFaultException, Exception;
	}

	private static Map<String, ServiceMethod> svcMethods = new HashMap<>();
	static {
		svcMethods.put("VerifyRTN", new ServiceMethod() {
			@Override
			public BooleanResult call(IRTNService svc, String token, String rtn, String country,
					Boolean includeInactives, Boolean includeWireSearch, Boolean includeHistory)
					throws ServerSOAPFaultException, Exception {
				return svc.verifyRTN(token, rtn, country, includeInactives, includeHistory);
			}
		});
		svcMethods.put("VerifyWire", new ServiceMethod() {
			@Override
			public BooleanResult call(IRTNService svc, String token, String rtn, String country,
					Boolean includeInactives, Boolean includeWireSearch, Boolean includeHistory)
					throws ServerSOAPFaultException, Exception {
				return svc.verifyWire(token, rtn, country, includeInactives, includeHistory);
			}
		});
	}

	@Override
	public void call(IRTNService svc, LogonProperties loginProps, String method, String[] paramsArray)
			throws StopProcesingException {
		String token = login(svc, loginProps);
		BooleanResult res = null;
		try {
			StringBuilder sb = new StringBuilder(500)
					.append("<RTN> <Country> <IncludeInactives> <includeWireSearch> <IncludeHistory>");
			if (paramsArray.length - 1 != 6) {
				printHelp(method, sb);
				return;
			}

			ServiceMethod svcMethod = svcMethods.get(method);
			Assert.notNull(svcMethod, "Usupported method");
			String rtn = null, country = null;
			Boolean includeInactives = null, includeWireSearch = null, includeHistory = null;
			
			rtn = paramsArray[1];
			country = paramsArray[2];
			includeHistory = Boolean.valueOf(paramsArray[paramsArray.length - 2]);
			includeWireSearch = Boolean.valueOf(paramsArray[paramsArray.length - 3]);
			includeInactives = Boolean.valueOf(paramsArray[paramsArray.length - 4]);

			if (rtn == null || country == null || includeInactives == null || includeWireSearch == null
					|| includeHistory == null) {
				printHelp(method, sb);
			} else {
				res = svcMethod.call(svc, token, rtn, country, includeInactives, includeWireSearch, includeHistory);
				Assert.notNull(res, "The returned BooleanResult is null");
			}

		} catch (ServerSOAPFaultException ex) {
			handleFault("\nError calling IRTNService.".concat(method), ex, false);
		} catch (Exception e) {
			handleError("\nError communicating to the IRTNService.".concat(method), e.getMessage(), true);
		}
		if (res != null) {
			System.out.println("\nThe IRTNService." + method + " returned " + res.isValue());
		}
		logoff(svc, token);
	}

}
