package com.as.lyons.wcf.svc.rtn;

import java.util.List;

import org.springframework.util.Assert;

import com.as.lyons.wcf.svc.rtn.client.IRTNService;
import com.as.lyons.wcf.svc.rtn.client.Institution;
import com.as.lyons.wcf.svc.rtn.client.InstitutionList;
import com.as.lyons.wcf.svc.rtn.client.InstitutionListResult;
import com.as.lyons.wcf.svc.rtn.client.InstitutionRoutingNumber;
import com.as.lyons.wcf.svc.rtn.client.InstitutionRoutingNumberList;
import com.as.lyons.wcf.svc.rtn.client.RoutingNumberHistory;
import com.as.lyons.wcf.svc.rtn.client.RoutingNumberHistoryList;
import com.sun.xml.ws.fault.ServerSOAPFaultException;

public class InstitutionListResultServiceCallBySwift extends AbstractRtnServiceCall{
	public void call(IRTNService svc, LogonProperties loginProps, String method, String[] params)
			throws StopProcesingException {
		String token = login(svc, loginProps);
		InstitutionList instNmLst = null;
		try {
			InstitutionListResult res = null;
			res = ParseAndInvokeGetInstitutionsDetailsBySwiftMethod(svc, token, params);
			if(res == null)
			{
				return;
			}
			instNmLst = res.getInstitutions();
	        reportResult(method, instNmLst);
			
		} catch (ServerSOAPFaultException ex) {
			handleFault("\nError calling IRTNService.".concat(method), ex, false);
		} catch (Exception e) {
			handleError("\nError communicating to the IRTNService.".concat(method), e.getMessage(), true);
		}
		logoff(svc, token);
	}

	private void reportResult(String method, InstitutionList res) {
		if (res == null) {
			return;
		}
		List<Institution> list = res.getInstitution();
		if (list == null || list.size() == 0) {
			message("\nThe RTN {} call returned no institutions", method);
			return;
		}
		int cnt = list.size();
		message("\nThe RTN {} call returned {} institution{}:", method, cnt, cnt == 1 ? "" : "s");
		for (Institution fi : list) {
			message("\t{}-{}-{}(id={}, mainBranchId={})", fi.getName(), fi.getCity(), fi.getState(),
					fi.getInstitutionId(), fi.getMainInstitutionId());

			InstitutionRoutingNumberList irnLst = fi.getRoutingNumbers();
			if (irnLst == null || irnLst.getInstitutionRoutingNumber().size() == 0) {
				message("\tNo routing numbers");
			} else {
				message("\tRouting numbers({}):", irnLst.getInstitutionRoutingNumber().size());
				for (InstitutionRoutingNumber irn : irnLst.getInstitutionRoutingNumber()) {
					message("\t\t{} ({})", irn.getRoutingNumber(), irn.getPurpose());
					RoutingNumberHistoryList rnhLst = irn.getHistory();
					if (rnhLst != null && rnhLst.getRoutingNumberHistory().size() > 0) {
						message("\t\tHistory({}):", rnhLst.getRoutingNumberHistory().size());
						for (RoutingNumberHistory rnh : rnhLst.getRoutingNumberHistory()) {
							message("\t\t\tRTN {}, purpose: {}", rnh.getRoutingNumber(), rnh.getPurpose());
						}
					}
				}
			}
		}
	}
	
	private InstitutionListResult ParseAndInvokeGetInstitutionsDetailsBySwiftMethod(IRTNService svc,  String token, String[] params) {
		log.info("started parsing");
		InstitutionListResult res = null;
		StringBuilder sb = new StringBuilder(500)
				.append("<Swift> <Country> <includeInactive> <includeRoutingNumberHistory>");
		if (params != null && params.length - 1 != 4) {
			printHelp("GetInstitutionsDetailsBySwift", sb);
			return null;
		}
		String swift = null, country = null;
		Boolean includeInactives = null, includeRoutingNumberHistory = null;
		// TODO parse parameters
		country = params[2];
		swift = params[1];
		includeRoutingNumberHistory = Boolean.valueOf(params[params.length - 1]);
		includeInactives = Boolean.valueOf(params[params.length - 2]);
		;
		log.info("done parsing");
		if (swift == null || country == null || includeInactives == null || includeRoutingNumberHistory == null) {
			printHelp("GetInstitutionsDetailsBySwift", sb);
			return null;
		} else {
			 res = svc.getInstitutionsDetailsBySwift(token, swift, country, includeInactives, includeRoutingNumberHistory);
			log.info("called service");
			Assert.notNull(res, "The returned InstitutionListResult is null");
			
	   }
		return res;
	}
}
