package com.as.lyons.wcf.svc.rtn;

import java.util.HashMap;
import java.util.Map;

import org.springframework.util.Assert;
import com.as.lyons.wcf.svc.rtn.client.IRTNService;
import com.as.lyons.wcf.svc.rtn.client.Institution;
import com.as.lyons.wcf.svc.rtn.client.InstitutionResult;
import com.as.lyons.wcf.svc.rtn.client.InstitutionRoutingNumber;
import com.as.lyons.wcf.svc.rtn.client.InstitutionRoutingNumberList;
import com.as.lyons.wcf.svc.rtn.client.RoutingNumberHistory;
import com.as.lyons.wcf.svc.rtn.client.RoutingNumberHistoryList;
import com.sun.xml.ws.fault.ServerSOAPFaultException;

/**
 * @author kredding
 *
 */

public class InstitutionResultServiceCall extends AbstractRtnServiceCall{
	private static interface ServiceMethod {
		InstitutionResult call(IRTNService svc, String token, int institutionId, Boolean includeHistory)
				throws ServerSOAPFaultException, Exception;
	}	
	private static Map<String, ServiceMethod> svcMethods = new HashMap<>();
	static {
		svcMethods.put("GetInstitutionDetailsById", new ServiceMethod() {
			@Override
			public InstitutionResult call(IRTNService svc, String token, int institutionId, Boolean includeHistory)
					throws ServerSOAPFaultException, Exception {
				return svc.getInstitutionDetailsById(token, institutionId, includeHistory);
			}
		});			
	}
	
	
	@Override
	public void call(IRTNService svc, LogonProperties loginProps, String method, String[] params)
			throws StopProcesingException {
		String token = login(svc, loginProps);
		Institution instNm = null;
		try {
			StringBuilder sb = new StringBuilder(500).append("<InstitutionId> <IncludeHistory>");
			if(params.length-1 != 2)
			{
				printHelp(method, sb);
			}
			ServiceMethod svcMethod = svcMethods.get(method);
			Boolean includeHistory  = null;			 
			Long longInstitutionId = Long.parseLong(params[1]);
			includeHistory = Boolean.valueOf(params[2]);	
			if (longInstitutionId == null || includeHistory == null ) {
				printHelp(method, sb);
			}
			else {
				Assert.notNull(svcMethod, "Usupported method");
				int institutionId = longInstitutionId.intValue();
				InstitutionResult res = svcMethod.call(svc, token, institutionId, includeHistory);
				Assert.notNull(res, "The returned InstitutionResult is null");
				instNm = res.getInstitution();
				Assert.notNull(instNm, "The returned Institution is null");
			}
			
		} catch (ServerSOAPFaultException ex) {
			handleFault("\nError calling IRTNService.".concat(method), ex, false);
		} catch (Exception e) {
			handleError("\nError communicating to the IRTNService.".concat(method), e.getMessage(), true);
		}
		reportResult(method, instNm);
		logoff(svc, token);
	}
	
	
	private void reportResult(String method, Institution fi) {
				
		if (fi == null) {
			message("\nThe RTN {} call returned no institutions", method);
			return;
		}		

		message("\t{}-{}-{}(id={}, mainBranchId={})", fi.getName(), fi.getCity(),fi.getState(), fi.getLegacyLyonsId(), fi.getMainInstitutionId());
			
		InstitutionRoutingNumberList irnLst = fi.getRoutingNumbers();
		if (irnLst == null || irnLst.getInstitutionRoutingNumber().size() == 0) {
			message("\tNo routing numbers");
		} else {
			message("\tRouting numbers({}):", irnLst.getInstitutionRoutingNumber().size());
			for (InstitutionRoutingNumber irn : irnLst.getInstitutionRoutingNumber()) {
				message("\t\t{} ({})", irn.getRoutingNumber(), irn.getPurpose());
				RoutingNumberHistoryList rnhLst = irn.getHistory();
				if (rnhLst != null && rnhLst.getRoutingNumberHistory().size() > 0) {
					message("\t\tHistory({}):", rnhLst.getRoutingNumberHistory().size());
					for (RoutingNumberHistory rnh : rnhLst.getRoutingNumberHistory()) {
						message("\t\t\t RTN {}, purpose: {}",
								rnh.getRoutingNumber(), rnh.getPurpose());
					}
				}
			}
		}		
	}
}
