package com.as.lyons.wcf.svc.rtn;
import java.util.HashMap;
import java.util.Map;
import org.springframework.util.Assert;
import com.as.lyons.wcf.svc.rtn.client.IRTNService;
import com.as.lyons.wcf.svc.rtn.client.Institution;
import com.as.lyons.wcf.svc.rtn.client.InstitutionResult;
import com.as.lyons.wcf.svc.rtn.client.InstitutionRoutingNumber;
import com.as.lyons.wcf.svc.rtn.client.InstitutionRoutingNumberList;
import com.as.lyons.wcf.svc.rtn.client.RoutingNumberHistory;
import com.as.lyons.wcf.svc.rtn.client.RoutingNumberHistoryList;
import com.sun.xml.ws.fault.ServerSOAPFaultException;

public class PrimaryInstitutionResultServiceCall extends AbstractRtnServiceCall {
	private static interface ServiceMethod {
		InstitutionResult call(IRTNService svc, String token, String rtn, String country, Boolean includeInactive,
				Boolean includeWireSearch, Boolean includeHistory) throws ServerSOAPFaultException, Exception;
	}

	private static Map<String, ServiceMethod> svcMethods = new HashMap<>();
	static {
		svcMethods.put("GetPrimaryInstitutionDetails", new ServiceMethod() {
			@Override
			public InstitutionResult call(IRTNService svc, String token, String rtn, String country,
					Boolean includeInactive, Boolean includeWireSearch, Boolean includeHistory)
					throws ServerSOAPFaultException, Exception {
				return svc.getPrimaryInstitutionDetails(token, rtn, country, includeInactive, includeWireSearch,
						includeHistory);
			}
		});
	}

	public void call(IRTNService svc, LogonProperties loginProps, String method, String[] params)
			throws StopProcesingException {
		String token = login(svc, loginProps);
		Institution instNm = null;
		try {
			log.info("started parsing");
			StringBuilder sb = new StringBuilder(500)
					.append("<RTN> <Country> <IncludeInactives> <includeHistory> <includeWireSearch>");

			if (params != null && params.length - 1 != 5) {
				printHelp(method, sb);
			}
			String rtn = null, country = null;
			Boolean includeInactive = null, includeWireSearch = null, includeHistory = null;
			// TODO parse parameters
			country = params[2];
			rtn = params[1];
			includeHistory = Boolean.valueOf(params[params.length - 1]);
			includeWireSearch = Boolean.valueOf(params[params.length - 2]);
			includeInactive = Boolean.valueOf(params[params.length - 3]);
			;
			log.info("done parsing");
			if (rtn == null || country == null || includeInactive == null || includeHistory == null
					|| includeWireSearch == null) {

				printHelp(method, sb);
			} else {
				log.info("Entered Primary instittuion details call ");
				ServiceMethod svcMethodsForPrimary = null;
				svcMethodsForPrimary = svcMethods.get(method);
				log.info(svcMethodsForPrimary.toString());
				log.info("Got primary method");
				InstitutionResult res = svcMethodsForPrimary.call(svc, token, rtn, country, includeInactive,
						includeWireSearch, includeHistory);
				instNm = res.getInstitution();
				log.info("called service");
				Assert.notNull(res, "The returned InstitutionResult is null");
			}
		} catch (ServerSOAPFaultException ex) {
			handleFault("\nError calling IRTNService.".concat(method), ex, false);
		} catch (Exception e) {
			handleError("\nError communicating to the IRTNService.".concat(method), e.getMessage(), true);
		}
		reportResult(method, instNm);
		logoff(svc, token);
	}

	private void reportResult(String method, Institution fi) {
		
		if (fi == null) {
			message("\nThe RTN {} call returned no institutions", method);
			return;
		}		

		message("\t{}-{}-{}(id={}, mainBranchId={})", fi.getName(), fi.getCity(),fi.getState(), fi.getLegacyLyonsId(), fi.getMainInstitutionId());
			
		InstitutionRoutingNumberList irnLst = fi.getRoutingNumbers();
		if (irnLst == null || irnLst.getInstitutionRoutingNumber().size() == 0) {
			message("\tNo routing numbers");
		} else {
			message("\tRouting numbers({}):", irnLst.getInstitutionRoutingNumber().size());
			for (InstitutionRoutingNumber irn : irnLst.getInstitutionRoutingNumber()) {
				message("\t\t{} ({})", irn.getRoutingNumber(), irn.getPurpose());
				RoutingNumberHistoryList rnhLst = irn.getHistory();
				if (rnhLst != null && rnhLst.getRoutingNumberHistory().size() > 0) {
					message("\t\tHistory({}):", rnhLst.getRoutingNumberHistory().size());
					for (RoutingNumberHistory rnh : rnhLst.getRoutingNumberHistory()) {
						message("\t\t\t RTN {}, purpose: {}",
								rnh.getRoutingNumber(), rnh.getPurpose());
					}
				}
			}
		}		
	}
}
